
# q-expansion of newform 108.5.c.b, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 108
weight = 5
poly_data = [6, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 18], 1]]

hecke_ring_character_values = [[55, [1, 0]], [29, [-1, 0]]]
aps_data = [[0, 0], [0, 0], [0, 1], [-31, 0], [0, -5], [-241, 0], [0, 5], [-271, 0], [0, 5], [0, 10], [-778, 0], [1079, 0], [0, -50], [-298, 0], [0, 75], [0, 70], [0, 65], [-2641, 0], [5609, 0], [0, -100], [7199, 0], [329, 0], [0, 30], [0, 185], [-15961, 0], [0, 440], [1649, 0], [0, -95], [-3238, 0], [0, -315], [-13858, 0], [0, 610], [0, 115], [25409, 0], [0, 140], [12569, 0], [21002, 0], [-16831, 0], [0, -675], [0, -570], [0, -500], [10079, 0], [0, 845], [6599, 0], [0, -55], [-57871, 0], [-1231, 0], [-60058, 0], [0, 840], [-3118, 0], [0, 820], [0, -1620], [-54121, 0], [0, -1730], [0, 570], [0, -940], [0, 2065], [-67231, 0], [34442, 0], [0, -1550], [-83578, 0], [0, -2035], [98102, 0], [0, -2165], [30839, 0], [0, 170], [-191551, 0], [-155401, 0], [0, -4380], [207239, 0], [0, 2620], [0, -2145], [62249, 0], [34679, 0], [198089, 0], [0, -200], [0, 1415], [202682, 0], [0, -920], [-105001, 0], [0, -435], [-252121, 0], [0, 7595], [-22678, 0], [-124858, 0], [0, 2010], [0, 1665], [33002, 0], [0, 4135], [63929, 0], [0, 7645], [0, -3030], [319769, 0], [0, -5305], [36302, 0], [0, -10365], [0, -3185], [0, 6815], [-195391, 0], [60119, 0], [311369, 0], [0, 5825], [0, -930], [0, 695], [-140911, 0], [-128161, 0], [0, 6995], [0, 9710], [0, 15530], [-373798, 0], [195329, 0], [-322201, 0], [0, 15325], [249449, 0], [-357751, 0], [0, -20], [-527098, 0], [0, -510], [0, -1390], [0, 7490], [-367801, 0], [93479, 0], [0, 9095], [0, 13980], [-19738, 0], [0, -4895], [-688081, 0], [0, -15090], [-355258, 0], [482522, 0], [571142, 0], [0, 4095], [-835711, 0], [406079, 0], [0, 7485], [358319, 0], [0, 11430], [182369, 0], [0, 27180], [0, -11020], [664742, 0], [0, 23585], [-745711, 0], [0, 8705], [-459001, 0], [0, -24570], [1119839, 0], [0, -24360], [-128311, 0], [0, -4945], [817679, 0], [0, -26105], [-757351, 0], [0, -18410], [706289, 0], [0, 6450], [1470422, 0], [0, 30600], [183959, 0], [0, -2485], [0, 34905], [0, -21075], [-561031, 0], [0, -22545], [0, -12430], [0, 33425], [-905671, 0], [-770518, 0]]
